<#  
.SYNOPSIS  
   	Configures the necessary prerequisites to install Lync 2010 on a Windows Server 2008 R2 server. Installs additional Microsoft and third party tools and utilities.
.DESCRIPTION  
    Installs all required Windows 2008 R2 components. Provides options for disabling TCP/IP v6, downloading latest Cumulative Update, etc.
.NOTES  
    Version      	
    	: v1.0 - 11/27/2009 - initial version
			: v3.0 - 02-2011 by Stle Hansen (http://msunified.net)
			: v4.0 - 04-06-2011
				added Visual C++ option
				added Stress and Performance Tool
				added Disable IPv6
				added "IM an Expert"
				added Windows Update
				lots of code cleanup
			: v4.1 - 05-01-2011
				added group membership check (checks for local admin, CSAdmin* and RTCUniv* only)
				added 04-20-2011 "Lync Server update download"
				added chm file download and install & shortcut
				added standalone AV server prereq option
				added PowerShell transcript
				added backup current config
			: v4.2 - 05-26-2011
				updated function to allow downloading files to different folders and different names
				added OWA / Lync integration feature
			: v5.0 - 07-25-2011
				added "Find Lync Versions" and related .Net 4 installer
				added functions for creating shorcuts and unzipping files
				added function for "Windows Media Format Runtime" (per http://support.microsoft.com/kb/2522454)
				some variable cleanup
				some error checking
				added function to run installation routines. This makes it easier to do consistent screen output (menu)
				cleaned up variable removal when exiting
				Installation of Silverlight
				Updated installation of Visuall C++ Redistributable to latest version (per MS11-025)
				added option to enable federation with Office365
		Wish list					: variable cleanup
											  Better error trapping
											  Better menu display (separate options into pre-install/post-install sections
											  Check for elevated rights before attempting tasks that require them
											  phone firmware updates
											  add loopback adapter and configure (for when using HLB with DSR)
											  creation of file share on remote server	(currently works online on local host)									  
    Rights Required		: Local administrator on server; CsAdinistrator for some functions
    Sched Task Req'd	: No
    Lync Version			: 2010 (tested through CU2)
    Author       			: Pat Richard, Exchange MVP
    Email / Blog 			: pat@innervation.com 	http://exchangeblogs.com
    Dedicated Post		: http://www.ucblogs.net/blogs/exchange/archive/2011/04/27/Automated-prerequisite-installation-via-PowerShell-for-Lync-Server-2010-on-Windows-Server-2008-R2.aspx
    Disclaimer   			: You running this script means you won't blame me if this breaks your stuff.
    Info Stolen from 	: Anderson Patricio and Bhargav Shukla
    									: (DISM command lines) http://technet.microsoft.com/en-us/library/dd744382(WS.10).aspx
    									: (Auto-Documenting Script Variables) http://powershell.com/cs/blogs/tips/archive/2009/04/10/auto-documenting-script-variables.aspx
    									: (Test Internet connection) http://powershell.com/cs/blogs/tips/archive/2011/05/04/test-internet-connection.aspx
    									: (Unzipping files) http://pshscripts.blogspot.com/2008/07/unzip-fileps1.html
    									: (Create shortcuts) http://www.leedesmond.com/weblog/?p=60.
    									: (Module check) http://blogs.technet.com/b/heyscriptingguy/archive/2010/07/11/hey-scripting-guy-weekend-scripter-checking-for-module-dependencies-in-windows-powershell.aspx
    									: (file share) http://blogs.technet.com/b/heyscriptingguy/archive/2010/09/16/how-to-use-powershell-to-create-shared-folders-in-windows-7.aspx
											: (file share description) http://serverfault.com/questions/9011/creating-a-share-with-permissions-with-windows-powershell
.LINK  
	http://www.ucblogs.net/blogs/exchange/archive/2011/04/27/Automated-prerequisite-installation-via-PowerShell-for-Lync-Server-2010-on-Windows-Server-2008-R2.aspx
.EXAMPLE
	.\Set-Lync2010Features.ps1
.INPUTS
	None. You cannot pipe objects to this script.
#>
#Requires -Version 2.0
[string] $strFilenameTranscript = $MyInvocation.MyCommand.Name + " " + (hostname)+ " {0:yyyy-MM-dd hh-mmtt}.log" -f (Get-Date)
Start-Transcript -path .\$strFilenameTranscript | Out-Null
# $ErrorActionPreference = "SilentlyContinue"
# $ErrorActionPreference = "inquire"
$error.clear()
# Detect correct OS here and exit if no match (we intentionally truncate the last character to account for service packs)
if ((Get-WMIObject win32_OperatingSystem).Version -notmatch '6.1.760'){
	Write-Host "`nThis script requires a version of Windows Server 2008 R2, which this is not. Exiting...`n" -ForegroundColor Red
	Stop-Transcript
	Exit
} #end OS detection

Clear-Host
pushd
[string] $strTargetFolder = "c:\_Install"
[bool] $global:boolRebootRequired = $false
if ((Get-Module BitsTransfer).installed -eq $true){[bool] $boolWasInstalled = $true}else{[bool] $boolWasInstalled = $false}
[string] $strMenuOption = "None"
[bool] $boolHasInternetAccess = ([Activator]::CreateInstance([Type]::GetTypeFromCLSID([Guid]'{DCB00C01-570F-4A9B-8D69-199FDBA5723B}')).IsConnectedToInternet)
[string] $strMenu = @'

	**********************************
	Lync Server 2010 - Features script
	**********************************

	Please select an option from the list below.

	1) Front-End
	2) Director
	3) Monitoring/Archiving
	4) Communicator Web Access (OCS 2007 R2)
	5) Mediation/Edge/XMPP GW/AV Conferencing
	6) Install Unified Communications Managed API 3.0 Runtime (alpha)
	7) GroupChat
	8) Install Lync Server 2010 Resource Kit
	9) Install Lync Server 2010 Best Practices Analyzer
	10) Install Lync Server 2010 Stress and Performance Tool
	11) Disable IPv6
	12) Install telnet client
	13) Install Visual C++ 2008 Redistributable
	14) Install Lync 'IM an Expert' (alpha)
	15) Launch Windows Update
	16) Check group membership
	17) Download 04-20-2011 Lync Server 2010 updates & start updater program
	18) Install Microsoft Lync Server 2010 Documentation Help File (07-20-2011)
	19) Backup local Lync server configuration
	20) OWA/Lync integration (run on Exchange server) (alpha)
	21) Install 'Find Lync Versions' (includes .Net 4 installation)(beta)
	22) Install Microsoft Silverlight
	23) Install Windows Media Format Runtime
	24) ** Reserved for future feature **
	25) ** Reserved for future feature **
	26) Enable federation with Office365

	98) Restart the server
	99) Exit

Select an option.. [1-99]?
'@

function New-FileDownload ([string]$strSourceFile,[string]$strDestFolder,[string]$strDestFile) {
	# I should switch this to using a param block and pipelining from property name - just for consistency
	# I should clean up the display text to be consistent with other functions
	$error.clear()
	if (!($strDestFolder)){$strDestFolder = $strTargetFolder}
	Get-ModuleStatus -name BitsTransfer
	if (!($strDestFile)){[string] $strDestFile = $strSourceFile.Substring($strSourceFile.LastIndexOf("/") + 1)}
	if (Test-Path $strDestFolder){
		Write-Host "Folder: `"$strDestFolder`" exists."
	} else{
		Write-Host "Folder: `"$strDestFolder`" does not exist, creating..." -NoNewline
		New-Item $strDestFolder -type Directory
		Write-Host "Done! " -ForegroundColor Green
	}
	if (Test-Path "$strDestFolder\$strDestFile"){
		Write-Host "File: $strDestFile exists."
	}else{
		if ($boolHasInternetAccess){
			Write-Host "File: $strDestFile does not exist, downloading..." -NoNewLine
			Start-BitsTransfer -Source "$strSourceFile" -Destination "$strDestFolder\$strDestFile"
			Write-Host "Done! " -ForegroundColor Green
		}else{
			Write-Host "Internet access not detected. Please resolve and try again." -ForegroundColor red
		}
	}
} # end function New-FileDownload

function New-UnzippedFile	{
	param (
		[parameter(ValueFromPipeline=$true, ValueFromPipelineByPropertyName=$true, Mandatory=$true, HelpMessage="No zip file specified")] 
		[string]$ZipFile,
    [parameter(ValueFromPipeline=$true, ValueFromPipelineByPropertyName=$true, Mandatory=$true, HelpMessage="No file to unzip specified")] 
    [string]$UnzipFile,
    [parameter(ValueFromPipeline=$true, ValueFromPipelineByPropertyName=$true, Mandatory=$true, HelpMessage="No location to unzip file specified")] 
    [string]$UnzipFileLocation
	)
	$error.clear()
	Write-Host "Zip File........................................................[" -NoNewLine
	Write-Host "Unzipping" -ForegroundColor yellow -NoNewLine
	Write-Host "]" -NoNewLine
	if (Get-Item $zipfile){    
		$objShell = new-object -com shell.application
		# where the .zip is
		$FuncZipFolder = $objShell.namespace($ZipFile) 
		# the item in the zip
		$FuncFile = $FuncZipFolder.parsename($UnzipFile)      
		# where the item is to go
		$FuncTargetFolder = $objShell.namespace($UnzipFileLocation)       
		# do the copy of zipfile item to target folder
		$FuncTargetFolder.copyhere($FuncFile)
	}
	if ($error){
		Write-Host "`b`b`b`b`b`b`b`b`b`bfailed!" -ForegroundColor red -NoNewLine
	}else{
		Write-Host "`b`b`b`b`b`b`b`b`b`bdone!" -ForegroundColor green -NoNewLine
	}		
	Write-Host "]    "
} # end function New-UnzippedFile

function New-Shortcut	{
	param (
		[parameter(ValueFromPipeline=$true, ValueFromPipelineByPropertyName=$true, Mandatory=$true, HelpMessage="No shortcut name specified")] 
		[string]$ShortcutName,
		[parameter(ValueFromPipeline=$true, ValueFromPipelineByPropertyName=$true, Mandatory=$true, HelpMessage="No shortcut file specified")] 
		[string]$ShortcutFile,
		[parameter(ValueFromPipeline=$true, ValueFromPipelineByPropertyName=$true, Mandatory=$true, HelpMessage="No shortcut working directory specified")] 
		[string]$ShortcutWorkingDir,
		[parameter(ValueFromPipeline=$true, ValueFromPipelineByPropertyName=$true, Mandatory=$true, HelpMessage="No shortcut description specified")] 
		[string]$ShortcutDescription
	)
	$error.clear()
	# AllUsers start menu = "$env:ProgramData\Microsoft\Windows\Start Menu\Programs"
	Write-Host "Shortcut........................................................[" -NoNewLine
	Write-Host "Creating" -ForegroundColor yellow -NoNewLine
	Write-Host "]" -NoNewLine
	if (Get-Item $ShortcutFile){
		$wshshell = New-Object -ComObject WScript.Shell
		if (($ShortcutName.Substring($ShortcutName.Length  4, 4) -ne ".lnk") -and ($ShortcutName.Substring($ShortcutName.Length  4, 4) -ne ".url")){
			$ShortcutName += ".lnk"
		}
		$lnk = $wshshell.CreateShortcut($ShortcutName)				
		$lnk.TargetPath = $ShortcutFile
		#maps to Start in: on GUI
		$lnk.WorkingDirectory = $ShortcutWorkingDir
		#corresponds to the Comment field on the shortcut tab.
		$lnk.Description = $ShortcutDescription							
		$lnk.Save()
	}
	if ($error){
		Write-Host "`b`b`b`b`b`b`b`b`b`bfailed!" -ForegroundColor red -NoNewLine
	}else{
		Write-Host "`b`b`b`b`b`b`b`b`bcreated!" -ForegroundColor green -NoNewLine
	}		
	Write-Host "]    "
} # end function New-Shortcut

function New-ProgramInstallation	{
	param (
		[parameter(ValueFromPipeline=$true, ValueFromPipelineByPropertyName=$true, Mandatory=$true, HelpMessage="No installation file specified")] 
		[string]$InstallFile,
		[parameter(ValueFromPipeline=$true, ValueFromPipelineByPropertyName=$true, Mandatory=$true, HelpMessage="No installation switches specified")] 
		[string]$InstallSwitches
	)
	$error.clear()
	$strDestFile = $InstallFile.Substring($InstallFile.LastIndexOf("\") + 1)
	Write-Host "File: $strDestFile" -NoNewLine
	for($i=0;$i -lt (64 - ($strDestFile.length + 6));$i++){Write-Host "." -NoNewLine}
	Write-Host "[" -NoNewLine
	Write-Host "Installing" -ForegroundColor yellow -NoNewLine
	Write-Host "]" -NoNewLine
	$install = $InstallFile + " " + $InstallSwitches
	Invoke-Expression $install
	if ($error){
		Write-Host "`b`b`b`b`b`b`b`b`b`b`bFailed" -ForegroundColor Red -NoNewLine
	}else{
		Write-Host "`b`b`b`b`b`b`b`b`b`b`binstalled!" -ForegroundColor Green -NoNewLine
	}
	Write-Host "]      "
} # end function New-ProgramInstallation

function New-Share { 
	param (
		[parameter(ValueFromPipeline=$true, ValueFromPipelineByPropertyName=$true, Mandatory=$true, HelpMessage="No folder name specified")] 
		[string]$FolderName,
    [parameter(ValueFromPipeline=$true, ValueFromPipelineByPropertyName=$true, Mandatory=$true, HelpMessage="No share name specified")] 
    [string]$ShareName,
    [parameter(ValueFromPipeline=$true, ValueFromPipelineByPropertyName=$true, Mandatory=$false, HelpMessage="No description specified")] 
    [string]$Description
	)
	$error.clear()
	# Check for folder; Create it if it doesn't exist
	If (!(Test-Path $FolderName)) { 
		New-Item $FolderName -type Directory | Out-Null
	} 

	# Check for share; Create it if it doesn't exist
	$Shares=[WMICLASS]"WIN32_Share" 
	if (!(Get-WMIObject Win32_share -filter "name='$ShareName'")){
		$Shares.Create($FolderName,$ShareName,0,65535,$Description) | Out-Null
		if (!($error)){
			# the share was created
			return $true
		}	else	{
			# there was an error
			return $false
		}
	} else	{
		# the share already exists
		return $false
	}
} # end function New-Share 

function New-EventLogEntry	{
	param (
		[parameter(ValueFromPipeline=$true, ValueFromPipelineByPropertyName=$true, Mandatory=$true, HelpMessage="No event log text specified")] 
		[string]$EventLogText,
		[parameter(ValueFromPipeline=$true, ValueFromPipelineByPropertyName=$true, Mandatory=$true, HelpMessage="No event type specified")] 
		[ValidateSet("Error","Warning","Information","SuccessAudit","FailureAudit")]
		[string]$EventType,
		[parameter(ValueFromPipeline=$true, ValueFromPipelineByPropertyName=$true, Mandatory=$true, HelpMessage="No event source specified")] 
		[string]$EventSource,
		[parameter(ValueFromPipeline=$true, ValueFromPipelineByPropertyName=$true, Mandatory=$true, HelpMessage="No event log specified")]
		[ValidateSet("Application","Security","Setup","System","Hardware Events","Internet Explorer","Key Management Service","Lync Server")]
		[string]$EventLog,
		[parameter(ValueFromPipeline=$true, ValueFromPipelineByPropertyName=$true, Mandatory=$true, HelpMessage="No event ID specified")] 
		[ValidateRange(0,65535)] 
		[int]$EventID
	)
	$objEvt 					=	new-object System.Diagnostics.EventLog("System")
	# $objEvt.Source	=	$strScriptName
	$objEvt.Source 		=	$EventSource
	# to see all possible event types: [enum]::getvalues([system.diagnostics.eventlogentrytype])
	# $infoevent=[System.Diagnostics.EventLogEntryType]::Information
	$objEvt.WriteEntry($EventLogText,$EventType,$eventID)
} # end function New-EventLogEntry

function Install-WindowsMediaFormatRuntime	{
	# Requires local admin	
	# required if installing Lync on Front End servers running Server 2008 SP1
	# If this isn't done BEFORE installing Lync, an installation error may occur as per
	# http://support.microsoft.com/kb/2522454
	if ((Get-WMIObject win32_OperatingSystem).Version -eq '6.1.7601'){
		$error.clear()
		New-ProgramInstallation -InstallFile "$env:systemroot\system32\dism.exe" -InstallSwitches "/online /add-package /packagepath:$env:windir\servicing\Packages\Microsoft-Windows-Media-Format-Package~31bf3856ad364e35~amd64~~6.1.7601.17514.mum /ignorecheck /NoRestart /quiet /logpath:$strTargetFolder\dism.log"
		$global:boolRebootRequired = $true
	}
} # end function Install-WindowsMediaFormatRuntime

function Install-TelnetClient	{
	Get-ModuleStatus -name "ServerManager"
	if ((Get-WindowsFeature Telnet-Client).installed -eq $false){
		$a = New-Object -comobject wscript.shell 
		$intAnswer = $a.popup("Do you want to include Telnet client?",0,"Telnet client",4) 
		if ($intAnswer -eq 6) { 
			#yes
			# Get-ModuleStatus -name "ServerManager"
			#if ((Get-WindowsFeature Telnet-Client).installed -eq $false){
				Add-WindowsFeature Telnet-Client | Out-Null
			#} else {
				#Write-Host "Telnet client already installed"
			#}
		} else { 
			#no
			# $a.popup("You answered no.") 
		} 
	}
} # end function Install-TelnetClient

function Install-DocViewer	{
	$a = new-object -comobject wscript.shell 
	$intAnswer = $a.popup("Lync Resource Kit documentation is based on Microsoft Word. Would you like to install the free Word viewer?",0,"Word Viewer",4) 
	if ($intAnswer -eq 6) { 
		#yes
		if (!(Get-Item "HKLM:\SOFTWARE\Wow6432Node\Microsoft\Windows\CurrentVersion\Uninstall\{90850409-6000-11D3-8CFE-0150048383C9}" -ErrorAction SilentlyContinue)){
			New-FileDownload "http://download.microsoft.com/download/6/a/6/6a689355-b155-4fa7-ad8a-dfe150fe7ac6/wordview_en-us.exe"
			New-ProgramInstallation -InstallFile "$strTargetFolder\wordview_en-us.exe" -InstallSwitches "/q"
		}
		if (!(Get-Item "HKLM:\SOFTWARE\Wow6432Node\Microsoft\Windows\CurrentVersion\Uninstall\{90120000-0020-0409-0000-0000000FF1CE}" -ErrorAction SilentlyContinue)){
			New-FileDownload "http://download.microsoft.com/download/9/2/2/9222D67F-7630-4F49-BD26-476B51517FC1/FileFormatConverters.exe"
			New-ProgramInstallation -InstallFile "$strTargetFolder\FileFormatConverters.exe" -InstallSwitches "/q"
		}
	} else { 
  	#no
  	# $a.popup("You answered no.") 
	} 
} # end function Install-DocViewer

function Install-SilverLight	{
	# http://dwlpr.wordpress.com/2010/04/15/silverlight-4-final-download-direct-link/
	if (!(Get-Item "HKLM:\SOFTWARE\Wow6432Node\Microsoft\Windows\CurrentVersion\Uninstall\{89F4137D-6C26-4A84-BDB8-2E5A4BB71E00}" -ErrorAction SilentlyContinue)){
		# 4.0.50401.0 (doesn't include hotfix from KB 2512827)
		# New-FileDownload "http://download.microsoft.com/download/2/2/C/22CABA89-3580-4611-8E0D-56749D2120DF/runtime/Silverlight.exe"
		# 4.0.60531.0 (http://www.microsoft.com/download/en/details.aspx?displaylang=en&id=26488)
		New-FileDownload "http://download.microsoft.com/download/C/0/B/C0B404F0-F025-4156-95B0-164AD7CA07AA/runtime/Silverlight.exe"
		New-ProgramInstallation -InstallFile "$strTargetFolder\silverlight.exe" -InstallSwitches "/q"
	}
} # end function Install-SilverLight

function Install-VisualC++Redist	{
	if (!(Get-Item "HKLM:\SOFTWARE\Microsoft\Windows\CurrentVersion\Uninstall\{4B6C7001-C7D6-3710-913E-5BC23FCE91E6}" -ErrorAction SilentlyContinue)){		
		# 9.0.30729.4148
		# New-FileDownload "http://download.microsoft.com/download/9/7/7/977B481A-7BA6-4E30-AC40-ED51EB2028F2/vcredist_x64.exe"
		# New-ProgramInstallation -InstallFile "$strTargetFolder\vcredist_x64.exe" -InstallSwitches "/qb"		
		# should add update from KB 2538243 (MS11-025) install with /q (see http://www.microsoft.com/technet/security/bulletin/MS11-025.mspx)(http://www.microsoft.com/download/en/details.aspx?displaylang=en&id=26368)
		# 9.0.30729.6161
		New-FileDownload "http://download.microsoft.com/download/5/D/8/5D8C65CB-C849-4025-8E95-C3966CAFD8AE/vcredist_x64.exe"
		New-ProgramInstallation -InstallFile "$strTargetFolder\vcredist_x64.exe" -InstallSwitches "/qb"		
		# validate by checking HKLM:\SOFTWARE\Microsoft\DevDiv\VC\Servicing\9.0\RED\1033 for "Install" = dword:1
		Do {Start-Sleep -m 100}while(!(Get-ItemProperty HKLM:\SOFTWARE\Microsoft\DevDiv\VC\Servicing\9.0\RED\1033 -Name Install -ErrorAction SilentlyContinue))
		Write-Host "Bingo!"
	}
} # end function Install-VisualC++Redist

function Get-ModuleStatus { 
	param	(
		[parameter(ValueFromPipeline=$true, ValueFromPipelineByPropertyName=$true, Mandatory=$true, HelpMessage="No module name specified!")] 
		[string]$name
	)
	if(!(Get-Module -name "$name")) { 
		if(Get-Module -ListAvailable | ? {$_.name -eq "$name"}) { 
			Import-Module -Name "$name" 
			# module was imported
			return $true
		} else {
			# module was not available
			return $false
		}
	}else {
		# module was already imported
		# Write-Host "$name module already imported"
		return $true
	}
} # end function Get-ModuleStatus

function Remove-IPv6	{
	$error.clear()
	Write-Host "TCP/IP v6......................................................[" -NoNewLine
	Write-Host "removing" -ForegroundColor yellow -NoNewLine
	Write-Host "]" -NoNewLine
	Set-ItemProperty -path HKLM:\SYSTEM\CurrentControlSet\Services\Tcpip6\Parameters -name DisabledComponents -value 0xffffffff -type dword
	if ($error){
		Write-Host "`b`b`b`b`b`b`b`bfailed!" -ForegroundColor red -NoNewLine
	}else{
		Write-Host "`b`b`b`b`b`b`b`b`bdone!" -ForegroundColor green -NoNewLine
	}
	Write-Host "]    "
	$global:boolRebootRequired = $true
} # end function Remove-IPv6

function Remove-ScriptVariables($path) {  
	$result = Get-Content $path |  
	ForEach { if ( $_ -match '(\$.*?)\s*=') {      
			$matches[1]  | ? { $_ -notlike '*.*' -and $_ -notmatch 'result' -and $_ -notmatch 'env:'}  
		}  
	}  
	ForEach ($v in ($result | Sort-Object | Get-Unique)){		
		# Write-Host "Removing" $v.replace("$","")
		Remove-Variable ($v.replace("$","")) -ErrorAction SilentlyContinue
	}
} # end function Get-ScriptVariables

Do {
	if ($global:boolRebootRequired -eq $true){Write-Host "`t`t`t`t`t`t`t`t`t`n`t`t`t`tREBOOT REQUIRED!`t`t`t`n`t`t`t`t`t`t`t`t`t`n`t`tDO NOT INSTALL Lync BEFORE REBOOTING!`t`t`n`t`t`t`t`t`t`t`t`t" -ForegroundColor yellow}
	if ($strMenuOption -ne "None") {Write-Host "`nLast command: "$strMenuOption -BackgroundColor Yellow -ForegroundColor black}
	$strMenuOption = Read-Host $strMenu

	switch ($strMenuOption) {
		1 { # Front-End
			Get-ModuleStatus -name "ServerManager"
			Install-VisualC++Redist
			Install-TelnetClient
			Add-WindowsFeature NET-Framework-Core, msmq-server, msmq-directory, RSAT-ADDS, Web-Static-Content, Web-Default-Doc, Web-Http-Errors, Web-Asp-Net, Web-Net-Ext, Web-ISAPI-Ext, Web-ISAPI-Filter, Web-Http-Logging, Web-Log-Libraries, Web-Http-Tracing, Web-Windows-Auth, Web-Filtering, Web-Stat-Compression, Web-Mgmt-Console, Web-Scripting-Tools, Web-Client-Auth, Desktop-Experience | Out-Null
			Install-SilverLight
			Install-WindowsMediaFormatRuntime
			$global:boolRebootRequired = $true 
		}
		2 { # Director
			Get-ModuleStatus -name "ServerManager"
			Install-VisualC++Redist
			Install-TelnetClient
			Add-WindowsFeature NET-Framework-Core, msmq-server, msmq-directory, Web-Static-Content, Web-Default-Doc, Web-Http-Errors, Web-Asp-Net, Web-Net-Ext, Web-ISAPI-Ext, Web-ISAPI-Filter, Web-Http-Logging, Web-Log-Libraries, Web-Http-Tracing, Web-Windows-Auth, Web-Filtering, Web-Stat-Compression, Web-Mgmt-Console, Web-Scripting-Tools, Web-Client-Auth
			Install-SilverLight
			$global:boolRebootRequired = $true 
		}
		3 { # Monitoring/Archiving
			Get-ModuleStatus -name "ServerManager"
			Install-VisualC++Redist
			Install-TelnetClient
			Add-WindowsFeature NET-Framework-Core, msmq-server, msmq-directory 			
		}
		4 { # Communicator Web Access (OCS 2007 R2)
			Get-ModuleStatus -name "ServerManager"
			Install-TelnetClient
			Add-WindowsFeature NET-Framework-Core, web-server, web-http-redirect, web-asp-net, web-http-logging, web-log-libraries, web-request-monitor, web-http-tracing, web-basic-auth, web-windows-auth, web-mgmt-console, web-scripting-tools, web-mgmt-compat, was-process-model, was-net-environment, rsat-adds
			$global:boolRebootRequired = $true 
		}
		5 { # Mediation/Edge/XMPP GW/AV Conferencing
			Get-ModuleStatus -name "ServerManager"
			Install-VisualC++Redist
			Install-TelnetClient
			Add-WindowsFeature NET-Framework-Core, msmq-server, msmq-directory 
		}
		6 { # Unified Communications Managed API 3.0 Runtime
			New-FileDownload "http://download.microsoft.com/download/3/B/1/3B13662A-8445-4238-A685-F15899032755/UcmaRuntimeSetup.exe"
			New-ProgramInstallation -InstallFile "$strTargetFolder\UcmaRuntimeSetup.exe" -InstallSwitches "/qb"
		}
		7 { # GroupChat
			Get-ModuleStatus -name "ServerManager"
			Install-TelnetClient
			Add-WindowsFeature NET-Framework-Core, web-server, web-mgmt-compat, web-asp-net, msmq-server, msmq-directory, rsat-adds
			$global:boolRebootRequired = $true 
		}
		8 { # Lync Server 2010 Resource Kit
			if (!(Get-Item "HKLM:\SOFTWARE\Microsoft\Windows\CurrentVersion\Uninstall\{9C44E652-CBBC-4E1C-8172-21A7D46CAB85}" -ErrorAction SilentlyContinue)){
				$error.clear()
				New-FileDownload "http://download.microsoft.com/download/1/8/A/18ADD884-E16F-4AD3-8422-FC80073439AA/OCSReskit.msi"
				Get-ModuleStatus -name "ServerManager"
				Add-WindowsFeature NET-Framework-Core 
				New-ProgramInstallation -InstallFile "$strTargetFolder\OCSReskit.msi" -InstallSwitches "/qb"
			}
			Install-DocViewer					
		}
		9 { # Lync Server 2010 Best Practices Analyzer
			if (!(Get-Item "HKLM:\SOFTWARE\Microsoft\Windows\CurrentVersion\Uninstall\{88033F09-23A0-44EC-9063-7CBAA4BE4975}" -ErrorAction SilentlyContinue)){
				$error.clear()
				New-FileDownload "http://download.microsoft.com/download/3/3/8/338D2314-1D23-47C1-8BB4-80FD39B84EA6/RTCBPA.msi"
				Get-ModuleStatus -name "ServerManager"
				Add-WindowsFeature NET-Framework-Core 
				New-ProgramInstallation -InstallFile "$strTargetFolder\RTCBPA.msi" -InstallSwitches "/qb"
			}				
		}
		10 { # Lync Server 2010 Stress and Performance Tool
			if (!(Get-Item "HKLM:\SOFTWARE\Microsoft\Windows\CurrentVersion\Uninstall\{94FD6E73-38F6-4DBE-AA3B-E403678868C9}" -ErrorAction SilentlyContinue)){
				Get-ModuleStatus -name "ServerManager"
				Add-WindowsFeature NET-Framework-Core
				Install-VisualC++Redist				
				New-FileDownload "http://download.microsoft.com/download/7/1/E/71EC25F9-575C-47BF-A4C0-E5DFFD42AA56/CapacityPlanningTool.msi"
				New-ProgramInstallation -InstallFile "$strTargetFolder\CapacityPlanningTool.msi" -InstallSwitches "/qb"
			}
			Install-DocViewer
		}
		11 { # Disable IPv6
			Remove-IPv6 
		}
		12 { # telnet client
			Install-TelnetClient
			#Get-ModuleStatus -name "ServerManager"
			#if ((Get-WindowsFeature Telnet-Client).installed -eq $false){
			#	Add-WindowsFeature Telnet-Client
			#}
		}
		13 { # Visual C++ 2008 Redistributable
			Install-VisualC++Redist 
		}
		14 { # Lync 'IM an Expert'
			Get-ModuleStatus -name "ServerManager"
			Add-WindowsFeature NET-Framework-Core 
			New-FileDownload "http://download.microsoft.com/download/3/B/1/3B13662A-8445-4238-A685-F15899032755/UcmaRuntimeSetup.exe"
			New-ProgramInstallation -InstallFile "$strTargetFolder\UcmaRuntimeSetup.exe" -InstallSwitches "/q"
				
			New-FileDownload "http://download.microsoft.com/download/9/E/2/9E2CBE89-A25F-4158-A2FC-CD227FC4ED14/IManExpert.msi"
			New-ProgramInstallation -InstallFile "$strTargetFolder\IManExpert.msi" -InstallSwitches "/qb"
		}
		15 { # Windows Update
			Invoke-Expression "$env:windir\system32\wuapp.exe startmenu"
		}
		16 { # Check group membership
			Whoami /groups /fo list | findstr /I BUILTIN\Administrators; Whoami /groups /fo list | findstr /I Cs; Whoami /groups /fo list | findstr /I Rtc
		}
		17 { # Lync Server 2010 updates
			if (!(Get-Item "HKLM:\SOFTWARE\Microsoft\Windows\CurrentVersion\Uninstall\{9521B708-9D80-46A3-9E58-A74ACF4E343E}" -ErrorAction SilentlyContinue)){					
				New-FileDownload "http://download.microsoft.com/download/3/4/1/341C256C-0E74-4968-B6FA-EEA87600E283/LyncServerUpdateInstaller.exe"					
				New-ProgramInstallation -InstallFile "$strTargetFolder\LyncServerUpdateInstaller.exe" -InstallSwitches ""
			}
		}
		18	{ # Microsoft Lync Server 2010 Documentation Help File (07-20-2011)
			New-FileDownload "http://download.microsoft.com/download/E/F/B/EFBBCD0C-CC33-4E91-B921-05D47550B9BF/Lync_ITPro.exe"
			New-ProgramInstallation -InstallFile "$strTargetFolder\Lync_ITPro.exe" -InstallSwitches "-auto"
			# We need to sleep while the installation routine runs, otherwise creating the shortcut will fail because the target doesn't exist yet
			Do {Start-Sleep -m 100}while(Get-Process | ?{$_.ProcessName -match "Lync_ITPro"})
			New-Shortcut -ShortcutName "$env:public\Desktop\Lync Server 2010 Help Documentation.lnk" -ShortcutFile "C:\Program Files\Microsoft Lync Server 2010\Documentation\Lync_ITPro.chm" -ShortcutWorkingDir "C:\Program Files\Microsoft Lync Server 2010\Documentation" -ShortcutDescription "Lync Server 2010 Help Documentation"
		}
		19 { # Backup local Lync server configuration
			# http://blog.danovich.com.au/2010/12/30/scheduled-task-powershell-script-to-backup-lync-server-2010-config/
			[string] $FileNameConfigBackup = "$strTargetFolder\config backup " + (hostname)+ " {0:yyyy-MM-dd hh-mmtt}.zip" -f (Get-Date)
 			Export-CsConfiguration -Filename .\$FileNameConfigBackup -Force:$True
 			# Should also incorporate http://blogs.technet.com/b/uc_mess/archive/2011/03/17/lync_2d00_server_2d00_2010_2d00_backup_2d00_instructions.aspx
		}
		20 { 	# OWA/Lync integration
			# to do: determine if we're running within the Exchange management shell
			if ($env:ExchangeInstallPath){
				New-FileDownload "http://download.microsoft.com/download/C/D/6/CD65672F-E0D1-471B-8404-B7B63966A8AA/CWAOWASSPMain.msi"
				New-ProgramInstallation -InstallFile "$$strTargetFolder\CWAOWASSPMain.msi" -InstallSwitches "/qb"
				$InstallDir = "C:\Web Service Provider Installer Package"
				New-FileDownload "http://download.microsoft.com/download/C/2/7/C27C449E-76FB-4BBF-A899-A82C2BBEB47C/CWAOWASSP.msp" $InstallDir
				New-FileDownload "http://download.microsoft.com/download/3/6/9/3693C940-1B9A-4386-836F-A21C7F4AE9C6/UcmaRedist.msp" $InstallDir
				New-FileDownload "http://download.microsoft.com/download/D/9/6/D967F77F-D0BD-43FB-987B-6FBF7C82251B/UcmaRedist.msp" $InstallDir "UcmaRedist-R2.msp"
				New-ProgramInstallation -InstallFile "$$strTargetFolder\UcmaRedist.msi" -InstallSwitches "/qb"
				New-ProgramInstallation -InstallFile "$$strTargetFolder\UcmaRedist.msp" -InstallSwitches "/qb"
				New-ProgramInstallation -InstallFile "$$strTargetFolder\UcmaRedist-R2.msp" -InstallSwitches "/qb"
				New-ProgramInstallation -InstallFile "$$strTargetFolder\CWAOWASSP.msi" -InstallSwitches "/qb"
				New-ProgramInstallation -InstallFile "$$strTargetFolder\CWAOWASSP.msp" -InstallSwitches "/qb"
				[string] $LyncPool = Read-Host "Enter FQDN of Lync front end server/pool"
					
				[string] $CertThumbprint = (Get-ExchangeCertificate |  Where {$_.Services -ilike "*IIS*"}).Thumbprint
				[string] $CertSubject = (Get-ExchangeCertificate |  Where {$_.Services -ilike "*IIS*"}).Subject
				$CertSubject = $subject.substring(3,$subject.indexof(",")-3)
				Get-ExchangeServer (hostname) | Get-OWAVirtualDirectory | Set-OWAVirtualDirectory -InstantMessagingType OCS -InstantMessagingEnabled:$true -InstantMessagingCertificateThumbprint $CertThumbprint -InstantMessagingServerName $LyncPool
				Get-ExchangeServer (hostname) | Get-OWAVirtualDirectory | fl name,instant*
					
				# prompt user to create CSTrustedApp using CN from certificate
				Write-Host "Create CsTrustedApplication using the name $CertSubject"
			}else{
				Write-Host "Exchange not detected. Cannot complete." -ForegroundColor red
			}
 		}
		21 { # Find Lync Versions	
			if (!(Get-ItemProperty "HKLM:\Software\Microsoft\NET Framework Setup\NDP\v4\Full" -Name install -ErrorAction SilentlyContinue | Out-Null)){
				New-FileDownload "http://download.microsoft.com/download/9/5/A/95A9616B-7A37-4AF6-BC36-D6EA96C8DAAE/dotNetFx40_Full_x86_x64.exe"
				New-ProgramInstallation -InstallFile "$strTargetFolder\dotNetFx40_Full_x86_x64.exe" -InstallSwitches "/q"
			}
			# http://www.stumper66.com/software/lync.html
			New-FileDownload "http://www.stumper66.com/software/files/Find Lync Versions.zip"
			New-UnzippedFile -ZipFile "$strTargetFolder\Find Lync Versions.zip" -UnzipFile "Find Lync Versions.exe" -UnzipFileLocation "C:\Program Files\Microsoft Lync Server 2010\Administrative Tools"
			New-Shortcut -ShortcutName "$env:public\Desktop\Find Lync Versions" -ShortcutFile "C:\Program Files\Microsoft Lync Server 2010\Administrative Tools\Find Lync Versions.exe" -ShortcutWorkingDir "C:\Program Files\Microsoft Lync Server 2010\Administrative Tools" -ShortcutDescription "Display client version and connection info"
		}
		22 { # SilverLight
			Install-SilverLight			
		}
		23	{ # WindowsMediaFormatRuntime
			Install-WindowsMediaFormatRuntime
		}
		24	{ # Upload phone firmware update
			<#
				if (Get-ModuleStatus -name Lync){
					[string] $strPoolName = (Get-CsPool | ?{$_.services -match "registrar"}).identity
					Import-CsDeviceUpdate -Identity "service:WebServer:$strPoolName" -FileName $ofd.FileName
				}
			#>
		}
		25	{ # create file share
			# New-Share -FolderName "c:\LyncShare" -ShareName "LyncShare" -Description "Used by Lync server to store Address Book files, phone updates, and other important files."
		}
		26	{ # enable federation with Office365
			# also need to make sure federation itself is enabled
			$error.clear()
			Write-Host "Office365 federation............................................[" -NoNewLine
			Write-Host "configuring" -ForegroundColor yellow -NoNewLine
			Write-Host "]" -NoNewLine
			if (Get-ModuleStatus -name "Lync") {
				if(!(Get-CsHostingProvider | ? {$_.proxyFqdn -match "sipfed.online.lync.com"})){
					New-CSHostingProvider identity "Office365" ProxyFqdn "sipfed.online.lync.com" Enabled $true -ErrorAction SilentlyContinue | Out-Null
					if (!($error)){
						Write-Host "`b`b`b`b`b`b`b`b`b`b`b`bdone!" -ForegroundColor green -NoNewLine
					}else{
						Write-Host "`b`b`b`b`b`b`b`b`b`b`b`bFailed!" -ForegroundColor red -NoNewLine
					}
				} else	{
					Write-Host "`b`b`b`b`b`b`b`b`b`b`b`balready enabled!" -ForegroundColor green -NoNewLine
				}
			}
			Write-Host "]          "
		}
		98 { # Exit and restart
			if (($boolWasInstalled -eq $false) -and (Get-Module BitsTransfer)){
				Write-Host "Removing BitsTransfer..." -NoNewLine
				Remove-Module BitsTransfer
				Write-Host "removed!" -ForegroundColor Green
			}
			Write-Host "Exiting and rebooting..."
			New-EventLogEntry -EventType Information -EventId 70 -EventLogText "Server shutdown initiated. See transcript file $pwd.path\$strFilenameTranscript for more information." -EventSource $MyInvocation.MyCommand.Name -EventLog Application
			Stop-Transcript
			Restart-Computer
		}
		99 { # Exit
			if (($boolWasInstalled -eq $false) -and (Get-Module BitsTransfer)){
				Write-Host "Removing BitsTransfer..." -NoNewLine
				Remove-Module BitsTransfer
				Write-Host "removed!" -ForegroundColor Green
			}
			popd
			Remove-ScriptVariables($MyInvocation.MyCommand.Name)
			Write-Host "Exiting..."
		}
		default {
			Write-Host "You haven't selected any of the available options."
		}
	}
} while ($strMenuOption -ne 99)
Stop-Transcript | Out-Null